@@ -1,17 +1,17 @@
-__version__ = '3.0.14'
+__version__ = "3.1.7"
 
 import pathlib  # isort:skip
 
 root = pathlib.Path(__file__).parent  # isort:skip
-APP_NAME = 'SPFY'
+APP_NAME = "SPFY"
 
 import os  # isort:skip
 
-ENV = os.getenv(f'{APP_NAME.upper()}_ENV', 'config')  # isort:skip
+ENV = os.getenv(f"{APP_NAME.upper()}_ENV", "config")  # isort:skip
 
 import kick  # isort:skip
 
-kick.start(f'{APP_NAME.lower()}', config_variant=ENV)  # isort:skip
+kick.start(APP_NAME.lower(), config_variant=ENV)  # isort:skip
 
 from kick import config, logger  # isort:skip
 
@@ -19,7 +19,7 @@ Unsplash = None
 if config.unsplash.auth.client_id:
     from unsplash import Auth, Api
 
-    auth = Auth(** config.unsplash.auth)
+    auth = Auth(**config.unsplash.auth)
     Unsplash = Api(auth)
 from .client import SpotifyClient
 from .result import SpotifyResult
