@@ -2,9 +2,12 @@ import logging
 
 import aiohttp
 from oauthlib.common import urldecode, generate_token
+from oauthlib.oauth2.rfc6749.errors import TokenExpiredError
 
 from oauthlib.oauth2 import (  # isort:skip
-    TokenExpiredError, WebApplicationClient, InsecureTransportError, is_secure_transport
+    WebApplicationClient,
+    InsecureTransportError,
+    is_secure_transport,
 )
 
 log = logging.getLogger(__name__)
@@ -17,6 +20,9 @@ class TokenUpdated(Warning):
         self.token = token
 
 
+# pylint: disable=too-many-instance-attributes
+
+
 class OAuth2Session(aiohttp.ClientSession):
     """Versatile OAuth 2 extension to :class:`aiohttp.ClientSession`.
 
@@ -36,6 +42,7 @@ class OAuth2Session(aiohttp.ClientSession):
     """
 
     #  pylint: disable=too-many-arguments
+
     def __init__(
         self,
         client_id=None,
@@ -88,19 +95,28 @@ class OAuth2Session(aiohttp.ClientSession):
         # Allow customizations for non compliant providers through various
         # hooks to adjust requests and responses.
         self.compliance_hook = {
-            'access_token_response': set(),
-            'refresh_token_response': set(),
-            'protected_request': set(),
+            "access_token_response": set(),
+            "refresh_token_response": set(),
+            "protected_request": set(),
         }
 
+    # pylint: disable=arguments-differ
+
+    def __del__(self, *args, **kwargs):
+        if not self.closed:
+            if self._connector_owner:
+                self._connector.close()
+            self._connector = None
+        super().__del__(*args, **kwargs)
+
     def new_state(self):
         """Generates a state string to be used in authorizations."""
         try:
             self._state = self.state()
-            log.debug(f'Generated new state {self._state}.')
+            log.debug("Generated new state %s.", self._state)
         except TypeError:
             self._state = self.state
-            log.debug(f'Re-using previously supplied state {self._state}.')
+            log.debug("Re-using previously supplied state %s.", self._state)
         return self._state
 
     @property
@@ -162,16 +178,17 @@ class OAuth2Session(aiohttp.ClientSession):
         ), state
 
     #  pylint: disable=too-many-locals
+
     async def fetch_token(
         self,
         token_url,
         code=None,
         authorization_response=None,
-        body='',
+        body="",
         auth=None,
         username=None,
         password=None,
-        method='POST',
+        method="POST",
         timeout=None,
         headers=None,
         verify_ssl=True,
@@ -214,7 +231,7 @@ class OAuth2Session(aiohttp.ClientSession):
             code = self._client.code
             if not code:
                 raise ValueError(
-                    'Please supply either code or ' 'authorization_response parameters.'
+                    "Please supply either code or " "authorization_response parameters."
                 )
 
         body = self._client.prepare_request_body(
@@ -225,27 +242,28 @@ class OAuth2Session(aiohttp.ClientSession):
             password=password,
             **kwargs,
         )
-        client_id = kwargs.get('client_id', '')
+        client_id = kwargs.get("client_id", "")
         if auth is None:
             if client_id:
                 log.debug(
-                    f'Encoding client_id "{client_id}" with client_secret as Basic auth credentials.'
+                    'Encoding client_id "%s" with client_secret as Basic auth credentials.',
+                    client_id,
                 )
-                client_secret = kwargs.get('client_secret', '')
-                client_secret = client_secret if client_secret is not None else ''
+                client_secret = kwargs.get("client_secret", "")
+                client_secret = client_secret if client_secret is not None else ""
                 auth = aiohttp.BasicAuth(client_id, client_secret)
             elif username:
                 if password is None:
-                    raise ValueError('Username was supplied, but not password.')
+                    raise ValueError("Username was supplied, but not password.")
 
-                log.debug('Encoding username, password as Basic auth credentials.')
+                log.debug("Encoding username, password as Basic auth credentials.")
                 auth = aiohttp.BasicAuth(username, password)
         headers = headers or {
-            'Accept': 'application/json',
-            'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
+            "Accept": "application/json",
+            "Content-Type": "application/x-www-form-urlencoded;charset=UTF-8",
         }
         self.token = {}
-        if method.upper() == 'POST':
+        if method.upper() == "POST":
             req = self.post(
                 token_url,
                 data=dict(urldecode(body)),
@@ -255,8 +273,8 @@ class OAuth2Session(aiohttp.ClientSession):
                 verify_ssl=verify_ssl,
                 proxy=proxy,
             )
-            log.debug(f'Prepared fetch token request body {body}')
-        elif method.upper() == 'GET':
+            log.debug("Prepared fetch token request body %s", body)
+        elif method.upper() == "GET":
             # if method is not 'POST', switch body to querystring and GET
             req = self.get(
                 token_url,
@@ -267,24 +285,25 @@ class OAuth2Session(aiohttp.ClientSession):
                 verify_ssl=verify_ssl,
                 proxy=proxy,
             )
-            log.debug(f'Prepared fetch token request querystring {body}')
+            log.debug("Prepared fetch token request querystring %s", body)
         else:
-            raise ValueError('The method kwarg must be POST or GET.')
+            raise ValueError("The method kwarg must be POST or GET.")
 
         async with req as resp:
             text = await resp.text()
-            log.debug(f'Request to fetch token completed with status {resp.status}.')
-            log.debug(f'Request headers were {resp.request_info.headers}')
-            log.debug(f'Response headers were {resp.headers} and content {text}.')
+            log.debug("Request to fetch token completed with status %s.", resp.status)
+            log.debug("Request headers were %s", resp.request_info.headers)
+            log.debug("Response headers were %s and content %s.", resp.headers, text)
             log.debug(
-                f'Invoking {len(self.compliance_hook["access_token_response"])} token response hooks.'
+                "Invoking %s token response hooks.",
+                len(self.compliance_hook["access_token_response"]),
             )
-            for hook in self.compliance_hook['access_token_response']:
-                log.debug(f'Invoking hook {hook}.')
+            for hook in self.compliance_hook["access_token_response"]:
+                log.debug("Invoking hook %s.", hook)
                 resp = hook(resp)
             self._client.parse_request_body_response(text, scope=self.scope)
             self.token = self._client.token
-            log.debug(f'Obtained token {self.token}.')
+            log.debug("Obtained token %s.", self.token)
         return self.token
 
     def token_from_fragment(self, authorization_response):
@@ -303,7 +322,7 @@ class OAuth2Session(aiohttp.ClientSession):
         self,
         token_url,
         refresh_token=None,
-        body='',
+        body="",
         auth=None,
         timeout=None,
         headers=None,
@@ -324,22 +343,24 @@ class OAuth2Session(aiohttp.ClientSession):
         :return: A token dict
         """
         if not token_url:
-            raise ValueError('No token endpoint set for auto_refresh.')
+            raise ValueError("No token endpoint set for auto_refresh.")
 
         if not is_secure_transport(token_url):
             raise InsecureTransportError()
 
-        refresh_token = refresh_token or self.token.get('refresh_token')
-        log.debug(f'Adding auto refresh key word arguments {self.auto_refresh_kwargs}.')
+        refresh_token = refresh_token or self.token.get("refresh_token")
+        log.debug(
+            "Adding auto refresh key word arguments %s.", self.auto_refresh_kwargs
+        )
         kwargs.update(self.auto_refresh_kwargs)
         body = self._client.prepare_refresh_body(
             body=body, refresh_token=refresh_token, scope=self.scope, **kwargs
         )
-        log.debug(f'Prepared refresh token request body {body}')
+        log.debug("Prepared refresh token request body %s", body)
         if headers is None:
             headers = {
-                'Accept': 'application/json',
-                'Content-Type': ('application/x-www-form-urlencoded;charset=UTF-8'),
+                "Accept": "application/json",
+                "Content-Type": ("application/x-www-form-urlencoded;charset=UTF-8"),
             }
         async with self.post(
             token_url,
@@ -352,23 +373,25 @@ class OAuth2Session(aiohttp.ClientSession):
             proxy=proxy,
         ) as resp:
             text = await resp.text()
-            log.debug(f'Request to refresh token completed with status {resp.status}.')
-            log.debug(f'Response headers were {resp.headers} and content {text}.')
+            log.debug("Request to refresh token completed with status %s.", resp.status)
+            log.debug("Response headers were %s and content %s.", resp.headers, text)
             log.debug(
-                f'Invoking {len(self.compliance_hook["refresh_token_response"])} token response hooks.'
+                "Invoking %s token response hooks.",
+                len(self.compliance_hook["refresh_token_response"]),
             )
-            for hook in self.compliance_hook['refresh_token_response']:
-                log.debug(f'Invoking hook {hook}.')
+            for hook in self.compliance_hook["refresh_token_response"]:
+                log.debug("Invoking hook %s.", hook)
                 resp = hook(resp)
             self.token = self._client.parse_request_body_response(
                 text, scope=self.scope
             )
-            if 'refresh_token' not in self.token:
-                log.debug('No new refresh token given. Re-using old.')
-                self.token['refresh_token'] = refresh_token
+            if "refresh_token" not in self.token:
+                log.debug("No new refresh token given. Re-using old.")
+                self.token["refresh_token"] = refresh_token
         return self.token
 
     #  pylint: disable=arguments-differ
+
     async def _request(
         self,
         method,
@@ -386,13 +409,13 @@ class OAuth2Session(aiohttp.ClientSession):
 
         if self.token and not withhold_token:
             log.debug(
-                'Invoking %d protected resource request hooks.',
-                len(self.compliance_hook['protected_request']),
+                "Invoking %d protected resource request hooks.",
+                len(self.compliance_hook["protected_request"]),
             )
-            for hook in self.compliance_hook['protected_request']:
-                log.debug(f'Invoking hook {hook}.')
+            for hook in self.compliance_hook["protected_request"]:
+                log.debug("Invoking hook %s.", hook)
                 url, headers, data = hook(url, headers, data)
-            log.debug(f'Adding token {self.token} to request.')
+            log.debug("Adding token %s to request.", self.token)
             try:
                 url, headers, data = self._client.add_token(
                     url, http_method=method, body=data, headers=headers
@@ -401,13 +424,15 @@ class OAuth2Session(aiohttp.ClientSession):
             except TokenExpiredError:
                 if self.auto_refresh_url:
                     log.debug(
-                        f'Auto refresh is set, attempting to refresh at {self.auto_refresh_url}.'
+                        "Auto refresh is set, attempting to refresh at %s.",
+                        self.auto_refresh_url,
                     )
                     # We mustn't pass auth twice.
-                    auth = kwargs.pop('auth', None)
+                    auth = kwargs.pop("auth", None)
                     if client_id and client_secret and (auth is None):
                         log.debug(
-                            f'Encoding client_id "{client_id}" with client_secret as Basic auth credentials.'
+                            'Encoding client_id "%s" with client_secret as Basic auth credentials.',
+                            client_id,
                         )
                         auth = aiohttp.BasicAuth(client_id, client_secret)
                     token = await self.refresh_token(
@@ -415,7 +440,7 @@ class OAuth2Session(aiohttp.ClientSession):
                     )
                     if self.token_updater:
                         log.debug(
-                            f'Updating token to {token} using {self.token_updater}.'
+                            "Updating token to %s using %s.", token, self.token_updater
                         )
                         self.token_updater(token)
                         url, headers, data = self._client.add_token(
@@ -427,9 +452,9 @@ class OAuth2Session(aiohttp.ClientSession):
                 else:
                     raise
 
-        log.debug(f'Requesting url {url} using method {method}.')
-        log.debug(f'Supplying headers {headers} and data {data}')
-        log.debug(f'Passing through key word arguments {kwargs}.')
+        log.debug("Requesting url %s using method %s.", url, method)
+        log.debug("Supplying headers %s and data %s", headers, data)
+        log.debug("Passing through key word arguments %s.", kwargs)
         return await super(OAuth2Session, self)._request(
             method, url, headers=headers, data=data, **kwargs
         )
@@ -446,6 +471,6 @@ class OAuth2Session(aiohttp.ClientSession):
         or open an issue.
         """
         if hook_type not in self.compliance_hook:
-            raise ValueError(f'Hook type {hook_type} is not in {self.compliance_hook}.')
+            raise ValueError(f"Hook type {hook_type} is not in {self.compliance_hook}.")
 
         self.compliance_hook[hook_type].add(hook)
